--
--  Copyright (C) 2009-2010, AdaCore
--

with AUnit.Test_Suites;

package AUnit.Test_Cases.Tests.Suite is

   function Test_Suite return AUnit.Test_Suites.Access_Test_Suite;

end AUnit.Test_Cases.Tests.Suite;
