--
--  Copyright (C) 2008, AdaCore
--
with Operands.Ints; use Operands.Ints;

package Operations.Ints is

   function "+" (Op1, Op2 : Int) return Int;

   function "-" (Op1, Op2 : Int) return Int;

end Operations.Ints;
